@@ -1,8 +1,8 @@
 ;;; muse-blosxom.el --- publish a document tree for serving by (py)Blosxom
 
-;; Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
-;; Author: Michael Olson (mwolson AT gnu DOT org)
+;; Author: Michael Olson <mwolson@gnu.org>
 ;; Date: Wed, 23 March 2005
 
 ;; This file is part of Emacs Muse.  It is not part of GNU Emacs.
@@ -50,8 +50,19 @@
 ;; `hardcodedates.py' provides the second service.  Eventually it is
 ;; hoped that a blosxom plugin and script will be found/written.
 ;;
-;; Generating a project description
-;; --------------------------------
+;; Alternately, the pyblosxom metadate plugin may be used.  Set the
+;; value of muse-blosxom-use-metadate to non-nil to enable adding a
+;; #postdate directive to all published files.  You can do this by:
+;;
+;; M-x customize-variable RET muse-blosxom-use-metadate RET
+;;
+;; With the metadate plugin installed in pyblosxom, the date set in
+;; this directive will be used instead of the file's modification
+;; time.  The plugin is available at:
+;; http://pyblosxom.sourceforge.net/blog/registry/date/metadate
+;;
+;; Generating a Muse project entry
+;; -------------------------------
 ;;
 ;; Muse-blosxom has some helper functions to make specifying
 ;; muse-blosxom projects a lot easier.  An example follows.
@@ -93,6 +104,16 @@
 ;;    the title, but in lowercase letters and having special
 ;;    characters converted to underscores.  The title and date
 ;;    directives will be inserted automatically.
+;;
+;; Using tags
+;; ----------
+;;
+;; If you wish to keep all of your blog entries in one directory and
+;; use tags to classify your entries, set `muse-blosxom-use-tags' to
+;; non-nil.
+;;
+;; For this to work, you will need to be using the PyBlosxom plugin at
+;; http://pyblosxom.sourceforge.net/blog/registry/meta/Tags.
 
 ;;; Contributors:
 
@@ -104,6 +125,10 @@
 ;; Björn Lindström (bkhl AT elektrubadur DOT se) made many valuable
 ;; suggestions.
 
+;; Sasha Kovar (sasha AT arcocene DOT org) fixed
+;; muse-blosxom-new-entry when using tags and also implemented support
+;; for the #postdate directive.
+
 ;;; Code:
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -127,7 +152,14 @@ See `muse-blosxom' for more information."
   :group 'muse-blosxom)
 
 (defcustom muse-blosxom-header
-  "<lisp>(muse-publishing-directive \"title\")</lisp>\n"
+  "<lisp>(concat (muse-publishing-directive \"title\") \"\\n\"
+  (when muse-blosxom-use-metadate
+    (let ((date (muse-publishing-directive \"date\")))
+      (when date (concat \"#postdate \"
+                         (muse-blosxom-format-date date) \"\\n\"))))
+  (when muse-blosxom-use-tags
+    (let ((tags (muse-publishing-directive \"tags\")))
+      (when tags (concat \"#tags \" tags \"\\n\")))))</lisp>"
   "Header used for publishing Blosxom files.  This may be text or a filename."
   :type 'string
   :group 'muse-blosxom)
@@ -143,6 +175,30 @@ This is the top-level directory where your Muse blog entries may be found."
   :type 'directory
   :group 'muse-blosxom)
 
+(defcustom muse-blosxom-use-tags nil
+  "Determine whether or not to enable use of the #tags directive.
+
+If you wish to keep all of your blog entries in one directory and
+use tags to classify your entries, set `muse-blosxom-use-tags' to
+non-nil.
+
+For this to work, you will need to be using the PyBlosxom plugin
+at http://pyblosxom.sourceforge.net/blog/registry/meta/Tags."
+  :type 'boolean
+  :group 'muse-blosxom)
+
+(defcustom muse-blosxom-use-metadate nil
+  "Determine whether or not to use the #postdate directive.
+
+If non-nil, published entries include the original date (as specified
+in the muse #date line) which can be read by the metadate PyBlosxom
+plugin.
+
+For this to work, you will need to be using the PyBlosxom plugin
+at http://pyblosxom.sourceforge.net/blog/registry/date/metadate."
+  :type 'boolean
+  :group 'muse-blosxom)
+
 ;; Maintain (published-file . date) alist, which will later be written
 ;; to a timestamps file; not implemented yet.
 
@@ -150,26 +206,27 @@ This is the top-level directory where your Muse blog entries may be found."
 
 (defun muse-blosxom-update-page-date-alist ()
   "Add a date entry to `muse-blosxom-page-date-alist' for this page."
-  ;; Make current file be relative to base directory
-  (let ((rel-file
-         (concat
-          (file-name-as-directory
-           (or (muse-publishing-directive "category")
-               (file-relative-name
-                (file-name-directory
-                 (expand-file-name muse-publishing-current-file))
-                (file-truename muse-blosxom-base-directory))))
-          (file-name-nondirectory muse-publishing-current-file))))
-    ;; Strip the file extension
-    (when muse-ignored-extensions-regexp
-      (setq rel-file (save-match-data
-                       (and (string-match muse-ignored-extensions-regexp
-                                          rel-file)
-                            (replace-match "" t t rel-file)))))
-    ;; Add to page-date alist
-    (add-to-list
-     'muse-blosxom-page-date-alist
-     `(,rel-file . ,(muse-publishing-directive "date")))))
+  (when muse-publishing-current-file
+    ;; Make current file be relative to base directory
+    (let ((rel-file
+           (concat
+            (file-name-as-directory
+             (or (muse-publishing-directive "category")
+                 (file-relative-name
+                  (file-name-directory
+                   (expand-file-name muse-publishing-current-file))
+                  (file-truename muse-blosxom-base-directory))))
+            (file-name-nondirectory muse-publishing-current-file))))
+      ;; Strip the file extension
+      (when muse-ignored-extensions-regexp
+        (setq rel-file (save-match-data
+                         (and (string-match muse-ignored-extensions-regexp
+                                            rel-file)
+                              (replace-match "" t t rel-file)))))
+      ;; Add to page-date alist
+      (add-to-list
+       'muse-blosxom-page-date-alist
+       `(,rel-file . ,(muse-publishing-directive "date"))))))
 
 ;; Enter a new blog entry
 
@@ -181,6 +238,10 @@ should be allowed in a filename."
   (muse-replace-regexp-in-string (concat "[^-." muse-regexp-alnum "]")
                                  "_" (downcase title)))
 
+(defun muse-blosxom-format-date (date)
+  "Convert a date string to PyBlosxom metadate plugin format."
+  (apply #'format "%s-%s-%s %s:%s" (split-string date "-")))
+
 ;;;###autoload
 (defun muse-blosxom-new-entry (category title)
   "Start a new blog entry with given CATEGORY.
@@ -188,38 +249,52 @@ The filename of the blog entry is derived from TITLE.
 The page will be initialized with the current date and TITLE."
   (interactive
    (list
-    (completing-read "Category: "
-                     (mapcar 'list (muse-project-recurse-directory
-                                    muse-blosxom-base-directory)))
+    (if muse-blosxom-use-tags
+        (let ((tag "foo")
+              (tags nil))
+          (while (progn (setq tag (read-string "Tag (RET to continue): "))
+                        (not (string= tag "")))
+            (add-to-list 'tags tag t))
+          tags)
+      (completing-read "Category: "
+                       (mapcar 'list (muse-project-recurse-directory
+                                      muse-blosxom-base-directory))))
     (read-string "Title: ")))
   (let ((file (muse-blosxom-title-to-file title)))
     (muse-project-find-file
      file "blosxom" nil
-     (concat (directory-file-name muse-blosxom-base-directory)
-             "/" category)))
+     (if muse-blosxom-use-tags
+         (directory-file-name muse-blosxom-base-directory)
+       (concat (directory-file-name muse-blosxom-base-directory)
+               "/" category))))
   (goto-char (point-min))
-  (insert "#date " (format-time-string "%4Y-%2m-%2d-%2H-%2M")
-          "\n#category " category
-          "\n#title " title
-          "\n\n")
+  (insert "#date " (format-time-string "%Y-%m-%d-%H-%M")
+          "\n#title " title)
+  (if muse-blosxom-use-tags
+      (if (> (length category) 0)
+          (insert (concat "\n#tags " (mapconcat #'identity category ","))))
+    (unless (string= category "")
+      (insert (concat "\n#category " category))))
+  (insert "\n\n")
   (forward-line 2))
 
-;; Register the Blosxom Publisher
+;;; Register the Muse Blosxom Publisher
 
-(unless (assoc "blosxom-html" muse-publishing-styles)
-  (muse-derive-style "blosxom-html" "html"
-                     :suffix    'muse-blosxom-extension
-                     :link-suffix 'muse-html-extension
-                     :header    'muse-blosxom-header
-                     :footer    'muse-blosxom-footer
-                     :after     'muse-blosxom-update-page-date-alist)
+(muse-derive-style "blosxom-html" "html"
+                   :suffix    'muse-blosxom-extension
+                   :link-suffix 'muse-html-extension
+                   :header    'muse-blosxom-header
+                   :footer    'muse-blosxom-footer
+                   :after     'muse-blosxom-update-page-date-alist
+                   :browser   'find-file)
 
-  (muse-derive-style "blosxom-xhtml" "xhtml"
-                     :suffix    'muse-blosxom-extension
-                     :link-suffix 'muse-xhtml-extension
-                     :header    'muse-blosxom-header
-                     :footer    'muse-blosxom-footer
-                     :after     'muse-blosxom-update-page-date-alist))
+(muse-derive-style "blosxom-xhtml" "xhtml"
+                   :suffix    'muse-blosxom-extension
+                   :link-suffix 'muse-xhtml-extension
+                   :header    'muse-blosxom-header
+                   :footer    'muse-blosxom-footer
+                   :after     'muse-blosxom-update-page-date-alist
+                   :browser   'find-file)
 
 (provide 'muse-blosxom)
 
