@@ -1,6 +1,6 @@
 ;;; muse-project.el --- handle Muse projects
 
-;; Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
 ;; This file is part of Emacs Muse.  It is not part of GNU Emacs.
 
@@ -31,6 +31,8 @@
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(provide 'muse-project)
+
 (require 'muse)
 (require 'muse-publish)
 (require 'cus-edit)
@@ -179,6 +181,11 @@ Muse can make use of."
                         (list :tag "No chapters"
                               (const :tag ":nochapters" ":nochapters")
                               (const t))
+                        (list :tag "Project-level publishing function"
+                              (const :tag ":publish-project"
+                                     ":publish-project")
+                              (choice (function :tag "Function")
+                                      (sexp :tag "Unknown")))
                         (list :tag "Set variables"
                               (const :tag ":set" ":set")
                               (repeat (list :inline t
@@ -188,7 +195,7 @@ Muse can make use of."
                               (const :tag ":visit-link" ":visit-link")
                               (choice (function)
                                       (sexp :tag "Unknown")))))
-               (repeat :tag "Styles" :format "%{%t%}:\n%v%i\n\n"
+               (repeat :tag "Output styles" :format "%{%t%}:\n%v%i\n\n"
                        (set :tag "Style"
                             (list :inline t
                                   :tag "Publishing style"
@@ -232,18 +239,34 @@ when publishing files in that project."
 ;; Make it easier to specify a muse-project-alist entry
 
 (defcustom muse-project-ignore-regexp
-  (concat "\\`\\(.*\\.?#.*\\|.*,v\\|.*~\\|\\.\\.?\\|,.*\\)\\'\\|"
-          "/\\(CVS\\|RCS\\|\\.arch-ids\\|{arch}\\|,.*\\)\\(/\\|\\'\\)")
-  "A regexp matching files to be ignored in Wiki directories.
+  (concat "\\`\\(#.*#\\|.*,v\\|.*~\\|\\.\\.?\\|\\.#.*\\|,.*\\)\\'\\|"
+          "/\\(CVS\\|RCS\\|\\.arch-ids\\|{arch}\\|,.*\\|\\.svn\\|"
+          "\\.hg\\|\\.git\\|\\.bzr\\|_darcs\\)\\(/\\|\\'\\)")
+  "A regexp matching files to be ignored in Muse directories.
 
-You should set case-fold-search to nil before using this regexp
+You should set `case-fold-search' to nil before using this regexp
 in code."
   :type 'regexp
   :group 'muse-regexp)
 
+(defcustom muse-project-publish-private-files t
+  "If this is non-nil, files will be published even if their permissions
+are set so that no one else on the filesystem can read them.
+
+Set this to nil if you would like to indicate that some files
+should not be published by manually doing \"chmod o-rwx\" on
+them.
+
+This setting has no effect under Windows (that is, all files are
+published regardless of permissions) because Windows lacks the
+needed filesystem attributes."
+  :type 'boolean
+  :group 'muse-project)
+
 (defun muse-project-recurse-directory (base)
   "Recusively retrieve all of the directories underneath BASE.
 A list of these directories is returned.
+
 Directories starting with \".\" will be ignored, as well as those
 which match `muse-project-ignore-regexp'."
   (let ((case-fold-search nil)
@@ -260,22 +283,29 @@ which match `muse-project-ignore-regexp'."
                               (muse-project-recurse-directory file)))))
       list)))
 
-(defun muse-project-alist-styles (entry-dir output-dir style)
+(defun muse-project-alist-styles (entry-dir output-dir style &rest other)
   "Return a list of styles to use in a `muse-project-alist' entry.
 ENTRY-DIR is the top-level directory of the project.
 OUTPUT-DIR is where Muse files are published, keeping directory structure.
 STYLE is the publishing style to use.
 
+OTHER contains other definitions to add to each style.  It is optional.
+
 For an example of the use of this function, see
 `examples/mwolson/muse-init.el' from the Muse distribution."
-  (cons `(:base ,style :path ,(expand-file-name output-dir)
-                :include ,(concat "/" (file-name-nondirectory entry-dir)
-                                  "/[^/]+$"))
-        (mapcar (lambda (dir);
-                  `(:base ,style
-                          :path ,(expand-file-name dir output-dir)
-                          :include ,(concat "/" dir "/[^/]+$")))
-                (muse-project-recurse-directory entry-dir))))
+  (let ((fnd (file-name-nondirectory entry-dir)))
+    (when (string= fnd "")
+      ;; deal with cases like "foo/" that have a trailing slash
+      (setq fnd (file-name-nondirectory (substring entry-dir 0 -1))))
+    (cons `(:base ,style :path ,(expand-file-name output-dir)
+                  :include ,(concat "/" fnd "/[^/]+$")
+                  ,@other)
+          (mapcar (lambda (dir)
+                    `(:base ,style
+                            :path ,(expand-file-name dir output-dir)
+                            :include ,(concat "/" dir "/[^/]+$")
+                            ,@other))
+                  (muse-project-recurse-directory entry-dir)))))
 
 (defun muse-project-alist-dirs (entry-dir)
   "Return a list of directories to use in a `muse-project-alist' entry.
@@ -292,10 +322,17 @@ For an example of the use of this function, see
 (defvar muse-project-file-alist nil
   "This variable is automagically constructed as needed.")
 
+(defvar muse-project-file-alist-hook nil
+  "Functions that are to be exectuted immediately after updating
+`muse-project-file-alist'.")
+
 (defvar muse-current-project nil
   "Project we are currently visiting.")
 (make-variable-buffer-local 'muse-current-project)
 
+(defvar muse-current-output-style nil
+  "The output style that we are currently using for publishing files.")
+
 (defsubst muse-project (&optional project)
   "Resolve the given PROJECT into a full Muse project, if it is a string."
   (if (null project)
@@ -306,14 +343,45 @@ For an example of the use of this function, see
       (muse-assert (consp project))
       project)))
 
-(defsubst muse-project-page-file (page project &optional no-check-p)
+(defun muse-project-page-file (page project &optional no-check-p)
   "Return a filename if PAGE exists within the given Muse PROJECT."
   (setq project (muse-project project))
-  (cdr (assoc page (muse-project-file-alist project no-check-p))))
+  (let ((dir (file-name-directory page))
+        (expanded-path nil))
+    (when dir
+      (setq expanded-path (concat (expand-file-name
+                                   page
+                                   (file-name-directory (muse-current-file)))
+                                  (when muse-file-extension
+                                    (concat "." muse-file-extension))))
+      (setq page (file-name-nondirectory page)))
+    (let ((files (muse-collect-alist
+                  (muse-project-file-alist project no-check-p)
+                  page))
+          (matches nil))
+      (if dir
+          (catch 'done
+            (save-match-data
+              (dolist (file files)
+                (if (and expanded-path
+                         (string= expanded-path (cdr file)))
+                    (throw 'done (cdr file))
+                  (let ((pos (string-match (concat (regexp-quote dir) "\\'")
+                                           (file-name-directory (cdr file)))))
+                    (when pos
+                      (setq matches (cons (cons pos (cdr file))
+                                          matches)))))))
+            ;; if we haven't found an exact match, pick a candidate
+            (car (muse-sort-by-rating matches)))
+        (dolist (file files)
+          (setq matches (cons (cons (length (cdr file)) (cdr file))
+                              matches)))
+        (car (muse-sort-by-rating matches '<))))))
 
 (defun muse-project-private-p (file)
   "Return non-nil if NAME is a private page with PROJECT."
-  (unless muse-under-windows-p
+  (unless (or muse-under-windows-p
+              muse-project-publish-private-files)
     (setq file (file-truename file))
     (if (file-attributes file)  ; don't publish if no attributes exist
         (or (when (eq ?- (aref (nth 8 (file-attributes
@@ -336,6 +404,8 @@ For an example of the use of this function, see
                                        (not (string= muse-file-extension "")))
                               (concat "." muse-file-extension "\\'"))))
         (unless (or (string-match muse-project-ignore-regexp file)
+                    (string-match muse-project-ignore-regexp
+                                  (file-name-nondirectory file))
                     (file-directory-p file))
           (setcdr lnames
                   (cons (cons (muse-page-name file) file) nil))
@@ -349,12 +419,17 @@ For an example of the use of this function, see
       (dolist (file (directory-files
                      (file-name-directory path) t
                      (file-name-nondirectory path)))
-        (unless (string-match muse-project-ignore-regexp file)
+        (unless (or (string-match muse-project-ignore-regexp file)
+                    (string-match muse-project-ignore-regexp
+                                  (file-name-nondirectory file)))
           (setcdr lnames
                   (cons (cons (muse-page-name file) file) nil))
           (setq lnames (cdr lnames))))))
     (cdr names)))
 
+(defvar muse-updating-file-alist-p nil
+  "Make sure that recursive calls to `muse-project-file-alist' are bounded.")
+
 (defun muse-project-file-alist (&optional project no-check-p)
   "Return member filenames for the given Muse PROJECT.
 On UNIX, this list is only updated if one of the directories'
@@ -387,6 +462,7 @@ disk."
     ;; Either return the currently known list, or read it again from
     ;; disk
     (if (or (and no-check-p (cadr file-alist))
+            muse-updating-file-alist-p
             (not (or muse-under-windows-p
                      (null (cddr file-alist))
                      (null last-mod)
@@ -398,17 +474,20 @@ disk."
               muse-project-file-alist
               (cons file-alist muse-project-file-alist)))
       ;; Read in all of the file entries
-      (save-match-data
-        (setcar
-         (cdr file-alist)
-         (let* ((names (list t))
-                (pats (cadr project)))
-           (while pats
-             (if (symbolp (car pats))
-                 (setq pats (cddr pats))
-               (nconc names (muse-project-file-entries (car pats)))
-               (setq pats (cdr pats))))
-           (cdr names)))))))
+      (let ((muse-updating-file-alist-p t))
+        (prog1
+            (save-match-data
+              (setcar
+               (cdr file-alist)
+               (let* ((names (list t))
+                      (pats (cadr project)))
+                 (while pats
+                   (if (symbolp (car pats))
+                       (setq pats (cddr pats))
+                     (nconc names (muse-project-file-entries (car pats)))
+                     (setq pats (cdr pats))))
+                 (cdr names))))
+          (run-hooks 'muse-project-file-alist-hook))))))
 
 (defun muse-project-of-file (&optional pathname)
   "Determine which project the given PATHNAME relates to.
@@ -417,9 +496,14 @@ If PATHNAME is nil, the current buffer's filename is used."
       muse-current-project
     (unless pathname (setq pathname (muse-current-file)))
     (save-match-data
-      (when (and pathname
+      (when (and (stringp pathname)
+                 (not (string= pathname ""))
                  (not (let ((case-fold-search nil))
-                        (string-match muse-project-ignore-regexp pathname))))
+                        (or (string-match muse-project-ignore-regexp
+                                          pathname)
+                            (string-match muse-project-ignore-regexp
+                                          (file-name-nondirectory
+                                           pathname))))))
         (let* ((file (file-truename pathname))
                (dir  (file-name-directory file))
                (project-entry muse-project-alist)
@@ -438,11 +522,19 @@ If PATHNAME is nil, the current buffer's filename is used."
               (setq project-entry (cdr project-entry))))
           found)))))
 
+(defun muse-project-after-save-hook ()
+  "Update Muse's file-alist if we are saving a Muse file."
+  (let ((project (muse-project-of-file)))
+    (when project
+      (muse-project-file-alist project))))
+
+(add-hook 'after-save-hook 'muse-project-after-save-hook)
+
 (defun muse-read-project (prompt &optional no-check-p no-assume)
   "Read a project name from the minibuffer, if it can't be figured
   out."
   (if (null muse-project-alist)
-      (error "There are no Muse projects defined; see `muse-project-alist'.")
+      (error "There are no Muse projects defined; see `muse-project-alist'")
     (or (unless no-check-p
           (muse-project-of-file))
         (if (and (not no-assume)
@@ -484,7 +576,10 @@ first directory within the project's fileset is used."
             name (cons name (muse-project-page-file name project))))
     ;; If we're given a relative or absolute filename, open it as-is
     (if (and (car name)
-             (save-match-data (string-match muse-file-regexp (car name))))
+             (save-match-data
+               (or (string-match "\\`\\.+/" (car name))
+                   (string-match muse-file-regexp (car name))
+                   (string-match muse-image-regexp (car name)))))
         (setcdr name (car name))
       ;; At this point, name is (PAGE . FILE).
       (unless (cdr name)
@@ -496,56 +591,155 @@ first directory within the project's fileset is used."
                   (setq directory (car pats) pats nil)
                 (setq pats (cdr pats))))))
         (when directory
-          (let ((filename (expand-file-name
-                           (if (and muse-file-extension
-                                    (not (string= muse-file-extension "")))
-                               (concat (car name) "." muse-file-extension)
-                             (car name))
-                           directory)))
+          (let ((filename (expand-file-name (car name) directory)))
+            (when (and muse-file-extension
+                       (not (string= muse-file-extension ""))
+                       (not (file-exists-p (car name))))
+              (setq filename (concat filename "." muse-file-extension)))
             (unless (file-exists-p directory)
               (make-directory directory t))
             (setcdr name filename)))))
     ;; Open the file
     (if (cdr name)
         (funcall (or command 'find-file) (cdr name))
-      (error "There is no page %s in project %s."
+      (error "There is no page %s in project %s"
              (car name) project-name))))
 
+(defun muse-project-choose-style (closure test styles)
+  "Run TEST on STYLES and return first style where TEST yields non-nil.
+TEST should take two arguments.  The first is CLOSURE, which is
+passed verbatim.  The second if the current style to consider.
+
+If no style passes TEST, return the first style."
+  (or (catch 'winner
+        (dolist (style styles)
+          (when (funcall test closure style)
+            (throw 'winner style))))
+      (car styles)))
+
+(defun muse-project-choose-style-by-link-suffix (given-suffix style)
+  "If the given STYLE has a link-suffix that equals GIVEN-SUFFIX,
+return non-nil."
+  (let ((link-suffix (or (muse-style-element :link-suffix style)
+                         (muse-style-element :suffix style))))
+    (and (stringp link-suffix)
+         (string= given-suffix link-suffix))))
+
 (defun muse-project-applicable-styles (file styles &optional ignore-regexp)
   "Given STYLES, return a list of the ones that are considered for FILE.
 The name of a project may be used for STYLES."
   (when (stringp styles)
     (setq styles (cddr (muse-project styles))))
   (when (and file styles)
-    (let (used-styles)
+    (let ((used-styles nil))
       (dolist (style styles)
         (let ((include-regexp (muse-style-element :include style))
-              (exclude-regexp (muse-style-element :exclude style)))
+              (exclude-regexp (muse-style-element :exclude style))
+              (rating nil))
           (when (and (or ignore-regexp
                          (and (null include-regexp)
                               (null exclude-regexp))
                          (if include-regexp
-                             (string-match include-regexp file)
+                             (setq rating (string-match include-regexp file))
                            (not (string-match exclude-regexp file))))
                      (or (not (file-exists-p file))
                          (not (muse-project-private-p file))))
-            (add-to-list 'used-styles style t))))
-      used-styles)))
+            (setq used-styles (cons (cons rating style) used-styles)))))
+      (muse-sort-by-rating (nreverse used-styles)))))
+
+(defun muse-project-get-applicable-style (file styles)
+  "Choose a style from the STYLES that FILE can publish to.
+The user is prompted if several styles are found."
+  (muse-publish-get-style
+   (mapcar (lambda (style)
+             (cons (muse-get-keyword :base style) style))
+           (muse-project-applicable-styles file styles))))
+
+(defun muse-project-resolve-link (page local-style remote-styles)
+  "Return a published relative link from the output path of one file
+to another file.
+
+The best match for PAGE is determined by comparing the link
+suffix of the given local style and that of the remote styles.
+
+The remote styles are usually populated by
+`muse-project-applicable-styles'."
+  (let ((link-suffix (or (muse-style-element :link-suffix local-style)
+                         (muse-style-element :suffix local-style)))
+        remote-style)
+    (if (not (stringp link-suffix))
+        (setq remote-style (car remote-styles))
+      (setq remote-style (muse-project-choose-style
+                          link-suffix
+                          #'muse-project-choose-style-by-link-suffix
+                          remote-styles)))
+    (if (null remote-style)
+        page
+      (muse-publish-link-file
+       (let ((prefix (muse-style-element :base-url remote-style)))
+         (if prefix
+             (concat prefix page)
+           (file-relative-name (expand-file-name
+                                (file-name-nondirectory page)
+                                (muse-style-element :path remote-style))
+                               (expand-file-name
+                                (muse-style-element :path local-style)))))
+       remote-style))))
+
+(defun muse-project-current-output-style (&optional file project)
+  (or muse-current-output-style
+      (progn
+        (unless file (setq file (muse-current-file)))
+        (unless project (setq project (muse-project-of-file file)))
+        (car (muse-project-applicable-styles file (cddr project))))))
+
+(defun muse-project-link-page (page)
+  (let ((project (muse-project-of-file)))
+    (muse-project-resolve-link page
+                               (muse-project-current-output-style)
+                               (muse-project-applicable-styles
+                                (muse-project-page-file page project)
+                                (cddr project)))))
+
+(defun muse-project-publish-file-default (file style output-dir force)
+  ;; ensure the publishing location is available
+  (unless (file-exists-p output-dir)
+    (message "Creating publishing directory %s" output-dir)
+    (make-directory output-dir t))
+  ;; publish the member file!
+  (muse-publish-file file style output-dir force))
 
 (defun muse-project-publish-file (file styles &optional force ignore-regexp)
   (setq styles (muse-project-applicable-styles file styles ignore-regexp))
   (let (published)
     (dolist (style styles)
-      (let ((output-dir (muse-style-element :path style)))
-        ;; ensure the publishing location is available
-        (unless (file-exists-p output-dir)
-          (message "Creating publishing directory %s" output-dir)
-          (make-directory output-dir t))
-        ;; publish the member file!
-        (if (muse-publish-file file style output-dir force)
-            (setq published t))))
+      (if (or (not (listp style))
+              (not (cdr style)))
+          (muse-display-warning
+           (concat "Skipping malformed muse-project-alist style."
+                   "\nPlease double-check your configuration,"))
+        (let ((output-dir (muse-style-element :path style))
+              (muse-current-output-style style)
+              (fun (or (muse-style-element :publish style t)
+                       'muse-project-publish-file-default)))
+          (when (funcall fun file style output-dir force)
+            (setq published t)))))
     published))
 
+;;;###autoload
+(defun muse-project-publish-this-file (&optional force)
+  "Publish the currently-visited file according to `muse-project-alist',
+prompting if more than one style applies.
+
+If FORCE is given, publish the file even if it is up-to-date."
+  (interactive (list current-prefix-arg))
+  (let* ((style (muse-project-get-applicable-style
+                buffer-file-name (cddr muse-current-project)))
+         (output-dir (muse-style-element :path style)))
+    (unless (muse-publish-file buffer-file-name style output-dir force)
+      (message (concat "The published version is up-to-date; use"
+                       " C-u C-c C-t to force an update.")))))
+
 (defun muse-project-save-buffers (&optional project)
   (setq project (muse-project project))
   (map-y-or-n-p
@@ -576,22 +770,10 @@ The name of a project may be used for STYLES."
    (if (boundp 'save-some-buffers-action-alist)
        save-some-buffers-action-alist)))
 
-;;;###autoload
-(defun muse-project-publish (project &optional force)
+(defun muse-project-publish-default (project styles &optional force)
   "Publish the pages of PROJECT that need publishing."
-  (interactive (list (muse-read-project "Publish project: " nil t)
-                     current-prefix-arg))
   (setq project (muse-project project))
-  (let ((styles (cddr project))
-        (muse-current-project project)
-        published)
-    ;; determine the style from the project, or else ask
-    (unless styles
-      (setq styles (list (muse-publish-get-style))))
-    ;; prompt to save any buffers related to this project
-    (muse-project-save-buffers project)
-    ;; run hook before publishing begins
-    (run-hook-with-args 'muse-before-project-publish-hook project)
+  (let ((published nil))
     ;; publish all files in the project, for each style; the actual
     ;; publishing will only happen if the files are newer than the
     ;; last published output, or if the file is listed in
@@ -613,6 +795,28 @@ The name of a project may be used for STYLES."
       (message "No pages in %s need publishing at this time."
                (car project)))))
 
+;;;###autoload
+(defun muse-project-publish (project &optional force)
+  "Publish the pages of PROJECT that need publishing."
+  (interactive (list (muse-read-project "Publish project: " nil t)
+                     current-prefix-arg))
+  (setq project (muse-project project))
+  (let ((styles (cddr project))
+        (muse-current-project project))
+    ;; determine the style from the project, or else ask
+    (unless styles
+      (setq styles (list (muse-publish-get-style))))
+    (unless project
+      (error "Cannot find a project to publish"))
+    ;; prompt to save any buffers related to this project
+    (muse-project-save-buffers project)
+    ;; run hook before publishing begins
+    (run-hook-with-args 'muse-before-project-publish-hook project)
+    ;; run the project-level publisher
+    (let ((fun (or (muse-get-keyword :publish-project (cadr project) t)
+                   'muse-project-publish-default)))
+      (funcall fun project styles force))))
+
 (defun muse-project-batch-publish ()
   "Publish Muse files in batch mode."
   (let ((muse-batch-publishing-p t)
@@ -662,6 +866,4 @@ The name of a project may be used for STYLES."
         (if output-file
             (muse-delete-file-if-exists output-file))))))
 
-(provide 'muse-project)
-
 ;;; muse-project.el ends here
