@@ -1,6 +1,6 @@
 ;;; muse-texinfo.el --- publish entries to Texinfo format or PDF
 
-;; Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
 ;; This file is part of Emacs Muse.  It is not part of GNU Emacs.
 
@@ -39,7 +39,7 @@
   "Rules for marking up a Muse file as a Texinfo article."
   :group 'muse-publish)
 
-(defcustom muse-texinfo-process-natively t
+(defcustom muse-texinfo-process-natively nil
   "If non-nil, use the Emacs `texinfmt' module to make Info files."
   :type 'boolean
   :require 'texinfmt
@@ -97,12 +97,7 @@ It may contain <lisp> markup tags."
   :type 'string
   :group 'muse-texinfo)
 
-(defcustom muse-texinfo-markup-regexps
-  `(;; join together the parts of a list or table
-    (10000 ,(concat "@end \\(\\(multi\\)?table\\|itemize\\|enumerate\\)"
-                    "\n\\{1,2\\}"
-                    "@\\1.*\n+")
-           0 ""))
+(defcustom muse-texinfo-markup-regexps nil
   "List of markup rules for publishing a Muse page to Texinfo.
 For more on the structure of this list, see `muse-publish-markup-regexps'."
   :type '(repeat (choice
@@ -115,8 +110,8 @@ For more on the structure of this list, see `muse-publish-markup-regexps'."
   :group 'muse-texinfo)
 
 (defcustom muse-texinfo-markup-functions
-  '((anchor . muse-texinfo-markup-anchor)
-    (table . muse-texinfo-markup-table))
+  '((table . muse-texinfo-markup-table)
+    (heading . muse-texinfo-markup-heading))
   "An alist of style types to custom functions for that kind of text.
 For more on the structure of this list, see
 `muse-publish-markup-functions'."
@@ -124,12 +119,15 @@ For more on the structure of this list, see
   :group 'muse-texinfo)
 
 (defcustom muse-texinfo-markup-strings
-  '((image-with-desc . "@image{%s}")
-    (image-link      . "@image{%s}")
-    (url-with-image  . "@uref{%s, %s}")
-    (url-link        . "@uref{%s, %s}")
-    (internal-link   . "@ref{%s, %s}")
+  '((image-with-desc . "@center @image{%1%, , , %3%, %2%}@*\n@center %3%")
+    (image           . "@noindent @image{%s, , , , %s}")
+    (image-link      . "@uref{%s, %s.%s}")
+    (anchor-ref      . "@ref{%s, %s}")
+    (url             . "@uref{%s}")
+    (link            . "@ref{Top, , Overview, %s, %s}")
+    (link-and-anchor . "@ref{%2%, , %2%, %1%, %3%}")
     (email-addr      . "@email{%s}")
+    (anchor          . "@anchor{%s} ")
     (emdash          . "---")
     (comment-begin   . "@ignore\n")
     (comment-end     . "\n@end ignore\n")
@@ -157,19 +155,21 @@ For more on the structure of this list, see
     (end-verse-line  . "")
     (verse-space     . "@ @ ")
     (end-verse       . "\n@end display")
-    (begin-example   . "@example")
-    (end-example     . "@end example")
+    (begin-example   . "@example\n")
+    (end-example     . "\n@end example")
     (begin-center    . "@quotation\n")
     (end-center      . "\n@end quotation")
     (begin-quote     . "@quotation\n")
     (end-quote       . "\n@end quotation")
-    (begin-uli       . "@itemize @bullet\n@item\n")
+    (begin-uli       . "@itemize @bullet\n")
     (end-uli         . "\n@end itemize")
-    (begin-oli       . "@enumerate\n@item\n")
+    (begin-uli-item  . "@item\n")
+    (begin-oli       . "@enumerate\n")
     (end-oli         . "\n@end enumerate")
-    (begin-ddt       . "@table @strong\n@item ")
-    (start-dde       . "\n")
-    (end-ddt         . "\n@end table"))
+    (begin-oli-item  . "@item\n")
+    (begin-dl        . "@table @strong\n")
+    (end-dl          . "\n@end table")
+    (begin-ddt       . "@item "))
   "Strings used for marking up text.
 These cover the most basic kinds of markup, the handling of which
 differs little between the various styles."
@@ -184,32 +184,82 @@ differs little between the various styles."
   :type '(alist :key-type character :value-type string)
   :group 'muse-texinfo)
 
-(defun muse-texinfo-insert-anchor (anchor)
-  "Insert an anchor, either before the next word, or within a tag."
-  (unless (get-text-property (match-end 1) 'noemphasis)
-    (skip-chars-forward (concat muse-regexp-blank "\n"))
-    (when (looking-at "<\\([^ />]+\\)>")
-      (goto-char (match-end 0)))
-    (insert "@anchor{" anchor "}")))
+(defcustom muse-texinfo-markup-specials-url
+  '((?@ . "@@")
+    (?{ . "@{")
+    (?} . "@}")
+    (?, . "@comma{}"))
+  "A table of characters which must be represented specially.
+These are applied to URLs."
+  :type '(alist :key-type character :value-type string)
+  :group 'muse-texinfo)
 
-(defun muse-texinfo-markup-anchor ()
-  (save-match-data
-    (muse-texinfo-insert-anchor (match-string 2)))
-  (match-string 1))
+(defun muse-texinfo-decide-specials (context)
+  "Determine the specials to escape, depending on CONTEXT."
+  (cond ((memq context '(underline literal emphasis email url url-desc image))
+         muse-texinfo-markup-specials-url)
+        (t muse-texinfo-markup-specials)))
 
 (defun muse-texinfo-markup-table ()
-  (let* ((str (prog1
-                  (match-string 1)
-                (delete-region (match-beginning 0) (match-end 0))))
-         (fields (split-string str "\\s-*|+\\s-*")))
-    (insert "@multitable @columnfractions")
-    (dotimes (field (length fields))
-      (insert " " (number-to-string (/ 1.0 (length fields)))))
-    (insert "\n@item " (mapconcat 'identity fields " @tab "))
-    (insert "\n@end multitable")))
+  (let* ((table-info (muse-publish-table-fields (match-beginning 0)
+                                                (match-end 0)))
+         (row-len (car table-info))
+         (field-list (cdr table-info)))
+    (when table-info
+      (muse-insert-markup "@multitable @columnfractions")
+      (dotimes (field row-len)
+        (muse-insert-markup " " (number-to-string (/ 1.0 row-len))))
+      (dolist (fields field-list)
+        (let ((type (car fields)))
+          (unless (eq type 'hline)
+            (setq fields (cdr fields))
+            (if (= type 2)
+                (muse-insert-markup "\n@headitem ")
+              (muse-insert-markup "\n@item "))
+            (insert (car fields))
+            (setq fields (cdr fields))
+            (dolist (field fields)
+              (muse-insert-markup " @tab ")
+              (insert field)))))
+      (muse-insert-markup "\n@end multitable")
+      (insert ?\n))))
 
-(defun muse-texinfo-finalize-buffer ()
-  (goto-char (point-min))
+(defun muse-texinfo-remove-links (string)
+  "Remove explicit links from STRING, replacing them with the link
+description.
+
+If no description exists for the link, use the link itself."
+  (let ((start nil))
+    (while (setq start (string-match muse-explicit-link-regexp string
+                                     start))
+      (setq string
+            (replace-match (or (match-string 2 string)
+                               (match-string 1 string))
+                           t t string)))
+    string))
+
+(defun muse-texinfo-protect-wikiwords (start end)
+  "Protect all wikiwords from START to END from further processing."
+  (and (boundp 'muse-wiki-wikiword-regexp)
+       (featurep 'muse-wiki)
+       (save-excursion
+         (goto-char start)
+         (while (re-search-forward muse-wiki-wikiword-regexp end t)
+           (muse-publish-mark-read-only (match-beginning 0)
+                                        (match-end 0))))))
+
+(defun muse-texinfo-markup-heading ()
+  (save-excursion
+    (muse-publish-markup-heading))
+  (let* ((eol (muse-line-end-position))
+         (orig-heading (buffer-substring (point) eol))
+         (beg (point)))
+    (delete-region (point) eol)
+    ;; don't allow links to be published in headings
+    (insert (muse-texinfo-remove-links orig-heading))
+    (muse-texinfo-protect-wikiwords beg (point))))
+
+(defun muse-texinfo-munge-buffer ()
   (muse-latex-fixup-dquotes)
   (texinfo-insert-node-lines (point-min) (point-max) t)
   (texinfo-all-menus-update t))
@@ -224,6 +274,10 @@ differs little between the various styles."
              (eq emacs-major-version 21))
     (put 'documentencoding 'texinfo-format
          'texinfo-discard-line-with-args))
+  ;; Most versions of `texinfmt.el' do not support @headitem, so hack
+  ;; it in.
+  (unless (get 'headitem 'texinfo-format)
+    (put 'headitem 'texinfo-format 'texinfo-multitable-item))
   (muse-publish-transform-output
    file output-path final-target "Info"
    (function
@@ -246,34 +300,35 @@ differs little between the various styles."
                       output-path " " file))))))))
 
 (defun muse-texinfo-pdf-generate (file output-path final-target)
-  (muse-publish-transform-output
-   file output-path final-target "PDF"
-   (function
-    (lambda (file output-path)
-      (= 0 (shell-command (concat "texi2pdf -q --clean --output="
-                                  output-path " " file)))))))
+  (let ((muse-latex-pdf-program "pdftex")
+        (muse-latex-pdf-cruft '(".aux" ".cp" ".fn" ".ky" ".log" ".pg" ".toc"
+                                ".tp" ".vr")))
+    (muse-latex-pdf-generate file output-path final-target)))
 
-(unless (assoc "texi" muse-publishing-styles)
-  (muse-define-style "texi"
-                     :suffix    'muse-texinfo-extension
-                     :regexps   'muse-texinfo-markup-regexps
-                     :functions 'muse-texinfo-markup-functions
-                     :strings   'muse-texinfo-markup-strings
-                     :specials  'muse-texinfo-markup-specials
-                     :after     'muse-texinfo-finalize-buffer
-                     :header    'muse-texinfo-header
-                     :footer    'muse-texinfo-footer
-                     :browser   'find-file)
+;;; Register the Muse TEXINFO Publishers
 
-  (muse-derive-style "info" "texi"
-                     :final   'muse-texinfo-info-generate
-                     :osuffix 'muse-texinfo-info-extension
-                     :browser 'info)
+(muse-define-style "texi"
+                   :suffix    'muse-texinfo-extension
+                   :regexps   'muse-texinfo-markup-regexps
+                   :functions 'muse-texinfo-markup-functions
+                   :strings   'muse-texinfo-markup-strings
+                   :specials  'muse-texinfo-decide-specials
+                   :after     'muse-texinfo-munge-buffer
+                   :header    'muse-texinfo-header
+                   :footer    'muse-texinfo-footer
+                   :browser   'find-file)
 
-  (muse-derive-style "info-pdf" "texi"
-                     :final   'muse-texinfo-pdf-generate
-                     :osuffix 'muse-texinfo-pdf-extension
-                     :browser 'muse-texinfo-pdf-browse-file))
+(muse-derive-style "info" "texi"
+                   :final   'muse-texinfo-info-generate
+                   :link-suffix 'muse-texinfo-info-extension
+                   :osuffix 'muse-texinfo-info-extension
+                   :browser 'info)
+
+(muse-derive-style "info-pdf" "texi"
+                   :final   'muse-texinfo-pdf-generate
+                   :link-suffix 'muse-texinfo-pdf-extension
+                   :osuffix 'muse-texinfo-pdf-extension
+                   :browser 'muse-texinfo-pdf-browse-file)
 
 (provide 'muse-texinfo)
 
