@@ -1,8 +1,8 @@
 ;;; muse-xml.el --- publish XML files
 
-;; Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 
-;; Author: Michael Olson (mwolson AT gnu DOT org)
+;; Author: Michael Olson <mwolson@gnu.org>
 ;; Date: Sat 23-Jul-2005
 
 ;; This file is part of Emacs Muse.  It is not part of GNU Emacs.
@@ -50,6 +50,7 @@
 
 (require 'muse-publish)
 (require 'muse-regexps)
+(require 'muse-xml-common)
 
 (defgroup muse-xml nil
   "Options controlling the behavior of Muse XML publishing.
@@ -80,38 +81,20 @@ This may be text or a filename."
 (defcustom muse-xml-footer "
   <!-- Page published by Emacs Muse ends here -->
 </MUSE>\n"
-  "Footer used for publishing Xml XML files.
+  "Footer used for publishing XML files.
 This may be text or a filename."
   :type 'string
   :group 'muse-xml)
 
 (defcustom muse-xml-markup-regexps
-  `(;; Join together the parts of a table
-    (10000 ,(concat "  </t\\(body\\|head\\|foot\\)>\\s-*</table>"
-                    "\\([" muse-regexp-blank "]*\n\\)\\{0,2\\}"
-                    "[" muse-regexp-blank "]*"
-                    "<table[^>]*>\\s-*<t\\1>\n")
-           0 "")
-    (10100 ,(concat "</table>"
-                    "\\([" muse-regexp-blank "]*\n\\)\\{0,2\\}"
-                    "[" muse-regexp-blank "]*"
-                    "<table[^>]*>\n")
-           0 "")
-
-    ;; Join together the parts of a list
-    (10200 ,(concat "</list>"
-                    "\\([" muse-regexp-blank "]*\n\\)\\{0,2\\}"
-                    "[" muse-regexp-blank "]*"
-                    "<list[^>]*>\\s-*")
-           0 "")
-
-    ;; Beginning of doc, end of doc, or plain paragraph separator
-    (10300 ,(concat "\\(\n</\\(blockquote\\|format\\)>\\)?"
-                    "\\(\\(\n\\(["
-                    muse-regexp-blank
-                    "]*\n\\)+\\)\\|\\`\\s-*\\|\\s-*\\'\\)"
-                    "\\(<\\(blockquote\\|format type=\"center\"\\)>\n\\)?")
-           0 muse-xml-markup-paragraph))
+  `(;; Beginning of doc, end of doc, or plain paragraph separator
+    (10000 ,(concat "\\(\\(\n\\(?:[" muse-regexp-blank "]*\n\\)*"
+                    "\\([" muse-regexp-blank "]*\n\\)\\)"
+                    "\\|\\`\\s-*\\|\\s-*\\'\\)")
+           ;; this is somewhat repetitive because we only require the
+           ;; line just before the paragraph beginning to be not
+           ;; read-only
+           3 muse-xml-markup-paragraph))
   "List of markup rules for publishing a Muse page to XML.
 For more on the structure of this list, see `muse-publish-markup-regexps'."
   :type '(repeat (choice
@@ -133,12 +116,16 @@ For more on the structure of this list, see
   :group 'muse-xml)
 
 (defcustom muse-xml-markup-strings
-  '((image-with-desc . "<image href=\"%s\">%s</image>")
-    (image-link      . "<image href=\"%s\"></image>")
-    (url-with-image  . "<link type=\"image\" href=\"%s\">%s</link>")
-    (url-link        . "<link type=\"url\" href=\"%s\">%s</link>")
+  '((image-with-desc . "<image href=\"%s.%s\">%s</image>")
+    (image           . "<image href=\"%s.%s\"></image>")
+    (image-link      . "<link type=\"image\" href=\"%s\">%s.%s</link>")
+    (anchor-ref      . "<link type=\"url\" href=\"#%s\">%s</link>")
+    (url             . "<link type=\"url\" href=\"%s\">%s</link>")
+    (link            . "<link type=\"url\" href=\"%s\">%s</link>")
+    (link-and-anchor . "<link type=\"url\" href=\"%s#%s\">%s</link>")
     (email-addr      . "<link type=\"email\" href=\"%s\">%s</link>")
-    (emdash          . " -- ")
+    (anchor          . "<anchor id=\"%s\" />\n")
+    (emdash          . "%s--%s")
     (comment-begin   . "<!-- ")
     (comment-end     . " -->")
     (rule            . "<hr />")
@@ -180,28 +167,36 @@ For more on the structure of this list, see
     (end-center      . "\n</format></p>")
     (begin-quote     . "<blockquote>\n")
     (end-quote       . "\n</blockquote>")
-    (begin-uli       . "<list type=\"unordered\">\n<item>")
-    (end-uli         . "</item>\n</list>")
-    (begin-oli       . "<list type=\"ordered\">\n<item>")
-    (end-oli         . "</item>\n</list>")
-    (begin-ddt       . "<list type=\"definition\">\n<item><term>")
-    (start-dde       . "</term>\n<definition>")
-    (end-ddt         . "</definition>\n</item>\n</list>"))
+    (begin-quote-item . "<p>")
+    (end-quote-item  . "</p>")
+    (begin-uli       . "<list type=\"unordered\">\n")
+    (end-uli         . "\n</list>")
+    (begin-uli-item  . "<item>")
+    (end-uli-item    . "</item>")
+    (begin-oli       . "<list type=\"ordered\">\n")
+    (end-oli         . "\n</list>")
+    (begin-oli-item  . "<item>")
+    (end-oli-item    . "</item>")
+    (begin-dl        . "<list type=\"definition\">\n")
+    (end-dl          . "\n</list>")
+    (begin-dl-item   . "<item>\n")
+    (end-dl-item     . "\n</item>")
+    (begin-ddt       . "<term>")
+    (end-ddt         . "</term>")
+    (begin-dde       . "<definition>")
+    (end-dde         . "</definition>")
+    (begin-table     . "<table%s>\n")
+    (end-table       . "</table>")
+    (begin-table-row . "    <tr>\n")
+    (end-table-row   . "    </tr>\n")
+    (begin-table-entry . "      <%s>")
+    (end-table-entry . "</%s>\n"))
   "Strings used for marking up text.
 These cover the most basic kinds of markup, the handling of which
 differs little between the various styles."
   :type '(alist :key-type symbol :value-type string)
   :group 'muse-xml)
 
-(defcustom muse-xml-markup-specials
-  '((?\" . "&quot;")
-    (?\< . "&lt;")
-    (?\> . "&gt;")
-    (?\& . "&amp;"))
-  "A table of characters which must be represented specially."
-  :type '(alist :key-type character :value-type string)
-  :group 'muse-xml)
-
 (defcustom muse-xml-encoding-default 'utf-8
   "The default Emacs buffer encoding to use in published files.
 This will be used if no special characters are found."
@@ -209,51 +204,28 @@ This will be used if no special characters are found."
   :group 'muse-xml)
 
 (defcustom muse-xml-charset-default "utf-8"
-  "The default Xml XML charset to use if no translation is
+  "The default XML charset to use if no translation is
 found in `muse-xml-encoding-map'."
   :type 'string
   :group 'muse-xml)
 
-(defcustom muse-xml-encoding-map
-  '((iso-8859-1         . "iso-8859-1")
-    (iso-2022-jp        . "iso-2022-jp")
-    (utf-8              . "utf-8")
-    (japanese-iso-8bit  . "euc-jp")
-    (chinese-big5       . "big5")
-    (mule-utf-8         . "utf-8")
-    (chinese-iso-8bit   . "gb2312")
-    (chinese-gbk        . "gbk"))
-  "An alist mapping emacs coding systems to appropriate Xml charsets.
-Use the base name of the coding system (i.e. without the -unix)."
-  :type '(alist :key-type coding-system :value-type string)
-  :group 'muse-xml)
-
-(defun muse-xml-transform-content-type (content-type)
-  "Using `muse-xml-encoding-map', try and resolve an emacs
-coding system to an associated XML coding system. If no
-match is found, `muse-xml-charset-default' is used instead."
-  (let ((match (and (fboundp 'coding-system-base)
-                    (assoc (coding-system-base content-type)
-                           muse-xml-encoding-map))))
-    (if match
-        (cdr match)
-      muse-xml-charset-default)))
-
 (defun muse-xml-encoding ()
   (muse-xml-transform-content-type
    (or (and (boundp 'buffer-file-coding-system)
             buffer-file-coding-system)
-       muse-xml-encoding-default)))
+       muse-xml-encoding-default)
+   muse-xml-charset-default))
 
 (defun muse-xml-markup-paragraph ()
   (let ((end (copy-marker (match-end 0) t)))
     (goto-char (match-beginning 0))
     (when (save-excursion
             (save-match-data
-              (and (re-search-backward "<\\(/?\\)p[ >]"
-                                       nil t)
+              (and (re-search-backward "<\\(/?\\)p[ >]" nil t)
                    (not (string-equal (match-string 1) "/")))))
-      (insert "</p>"))
+      (when (get-text-property (1- (point)) 'end-list)
+        (goto-char (previous-single-property-change (1- (point)) 'end-list)))
+      (muse-insert-markup "</p>"))
     (goto-char end))
   (cond
    ((eobp)
@@ -262,78 +234,9 @@ match is found, `muse-xml-charset-default' is used instead."
    ((eq (char-after) ?\<)
     (when (looking-at (concat "<\\(format\\|code\\|link\\|image"
                               "\\|anchor\\|footnote\\)[ >]"))
-      (insert "<p>")))
+      (muse-insert-markup "<p>")))
    (t
-    (insert "<p>"))))
-
-(defun muse-xml-insert-anchor (anchor)
-  "Insert an anchor, either around the word at point, or within a tag."
-  (unless (get-text-property (match-end 1) 'noemphasis)
-    (skip-chars-forward (concat muse-regexp-blank "\n"))
-    (when (looking-at "<\\([^ />]+\\)>")
-      (goto-char (match-end 0)))
-    (insert "<anchor id=\"" anchor "\" />\n")))
-
-(defun muse-xml-markup-anchor ()
-  (save-match-data
-    (muse-xml-insert-anchor (match-string 2)))
-  (match-string 1))
-
-(defun muse-xml-markup-table ()
-  (let* ((str (prog1
-                  (match-string 1)
-                (delete-region (match-beginning 0) (match-end 0))))
-         (fields (split-string str "\\s-*|+\\s-*"))
-         (type (and (string-match "\\s-*\\(|+\\)\\s-*" str)
-                    (length (match-string 1 str))))
-         (part (cond ((= type 1) "tbody")
-                     ((= type 2) "thead")
-                     ((= type 3) "tfoot")))
-         (col (cond ((= type 1) "td")
-                    ((= type 2) "th")
-                    ((= type 3) "td"))))
-    (insert "<table>\n"
-            "  <" part ">\n"
-            "    <tr>\n")
-    (dolist (field fields)
-      (insert "      <" col ">" field "</" col ">\n"))
-    (insert "    </tr>\n"
-            "  </" part ">\n"
-            "</table>\n")))
-
-(defun muse-xml-fixup-tables ()
-  "Sort table parts."
-  (goto-char (point-min))
-  (let (last)
-    (while (re-search-forward "^<table[^>]*>$" nil t)
-      (unless (get-text-property (point) 'read-only)
-        (forward-line 1)
-        (save-restriction
-          (let ((beg (point)))
-            (narrow-to-region beg (and (re-search-forward "^</table>"
-                                                          nil t)
-                                       (match-beginning 0))))
-          (goto-char (point-min))
-          (let ((inhibit-read-only t))
-            (sort-subr nil
-                       (function
-                        (lambda ()
-                          (if (re-search-forward
-                               "^\\s-*<t\\(head\\|body\\|foot\\)>$" nil t)
-                              (goto-char (match-beginning 0))
-                            (goto-char (point-max)))))
-                       (function
-                        (lambda ()
-                          (if (re-search-forward
-                               "^\\s-*</t\\(head\\|body\\|foot\\)>$" nil t)
-                              (goto-char (match-end 0))
-                            (goto-char (point-max)))))
-                       (function
-                        (lambda ()
-                          (looking-at "\\s-*<t\\(head\\|body\\|foot\\)>")
-                          (cond ((string= (match-string 1) "head") 1)
-                                ((string= (match-string 1) "foot") 2)
-                                (t 3)))))))))))
+    (muse-insert-markup "<p>"))))
 
 (defun muse-xml-finalize-buffer ()
   (when (boundp 'buffer-file-coding-system)
@@ -341,20 +244,18 @@ match is found, `muse-xml-charset-default' is used instead."
       ;; make it agree with the default charset
       (setq buffer-file-coding-system muse-xml-encoding-default))))
 
-;; Register the Muse XML Publisher
+;;; Register the Muse XML Publisher
 
-(unless (assoc "xml" muse-publishing-styles)
-  (muse-define-style "xml"
-                     :suffix     'muse-xml-extension
-                     :regexps    'muse-xml-markup-regexps
-                     :functions  'muse-xml-markup-functions
-                     :strings    'muse-xml-markup-strings
-                     :specials   'muse-xml-markup-specials
-                     :before-end 'muse-xml-fixup-tables
-                     :after      'muse-xml-finalize-buffer
-                     :header     'muse-xml-header
-                     :footer     'muse-xml-footer
-                     :browser    'find-file))
+(muse-define-style "xml"
+                   :suffix     'muse-xml-extension
+                   :regexps    'muse-xml-markup-regexps
+                   :functions  'muse-xml-markup-functions
+                   :strings    'muse-xml-markup-strings
+                   :specials   'muse-xml-decide-specials
+                   :after      'muse-xml-finalize-buffer
+                   :header     'muse-xml-header
+                   :footer     'muse-xml-footer
+                   :browser    'find-file)
 
 (provide 'muse-xml)
 
